//
// MainViewController.swift
// VRPaymeTestApp
// 
// Created by VR Payment GmbH on 24.09.19
// Copyright © 2019 VR Payment GmbH. All rights reserved.
//

import UIKit
import VRPayme
import StoreKit

class MainViewController: UIViewController {

    #if OBJC
    var apiService: APIService = OBJCVRPaymeAPIService() as! APIService
    #else
    var apiService: APIService = VRPaymeAPIService()
    #endif

    private let transactionIDHint = "Transaktions-ID: %@"
    private let transactionIDMissingHint = "Noch keine Transaktions-ID zur Stornierung vorhanden"

    private var lastTranID: String?
    private var lastTranAmount: Int?
    @IBOutlet var cancelInfoLabel: UILabel!
    @IBOutlet var cancelTransactionButton: UIButton!
    @IBOutlet var dataClearingButton: UIButton!
    @IBOutlet var lastTransactionButton: UIButton!
    @IBOutlet var responseTextView: UITextView!

    override func viewDidLoad() {
        super.viewDidLoad()
        setupUserInterface()
    }

    @IBAction func cancelTransactionButtonPressed(_ sender: UIButton) {
        do {
            guard let tranID = lastTranID, let amount = lastTranAmount else {
                // Can't cancel transaction without the transaction id and the amount
                return
            }
            try apiService.performCancelation(identifier: tranID, amount: amount)
        } catch RequestBuilderError.missingVRPaymeApp {
            presentMissingVRPayMeAppAlert()
        } catch let error {
            updateResponseField(text: error.localizedDescription)
        }
    }

    @IBAction func dataClearingButtonPressed(_ sender: UIButton) {
        do {
            try apiService.performDataClearing()
        } catch RequestBuilderError.missingVRPaymeApp {
            presentMissingVRPayMeAppAlert()
        } catch let error {
            updateResponseField(text: error.localizedDescription)
        }
    }

    @IBAction func lastTransactionButtonPressed(_ sender: UIButton) {
        do {
            try apiService.performSync()
        } catch RequestBuilderError.missingVRPaymeApp {
            presentMissingVRPayMeAppAlert()
        } catch let error {
            updateResponseField(text: error.localizedDescription)
        }
    }
}

extension MainViewController {

    func updateResponseField(text: String) {
        responseTextView.text = text
    }

    func presentMissingVRPayMeAppAlert() {
        let title = "Benötigte App nicht installiert"
        updateResponseField(text: title)
        let message = "Sie benötigen für diese Aktion die VR pay:Me App. Bitte laden Sie sich diese herunter."
        let downloadActionTitle = "Herunterladen"
        let cancelActionTitle = "Abbruch"
        let alert = UIAlertController(title: title, message: message, preferredStyle: .alert)
        let downloadAction = UIAlertAction(title: downloadActionTitle, style: .default) { _ in
            self.downloadVRPayMeApp()
        }
        let cancelAction = UIAlertAction(title: cancelActionTitle, style: .default) { _ in
            self.updateResponseField(text: "")
        }
        alert.addAction(cancelAction)
        alert.addAction(downloadAction)
        present(alert, animated: true, completion: nil)
    }

    func downloadVRPayMeApp() {
        let controller = SKStoreProductViewController()
        controller.delegate = self
        let parameters = [SKStoreProductParameterITunesItemIdentifier: "1451816109"]
        updateResponseField(text: "Benötigte App wird installiert bitte warten.")
        controller.loadProduct(withParameters: parameters) { (success, _) in
            if success {
                self.present(controller, animated: true, completion: {
                    self.updateResponseField(text: "")
                })
            } else {
                self.updateResponseField(text: "Download ist fehlgeschlagen, bitte installieren Sie die VR pay:me App manuell aus dem App Store")
            }
        }
    }

    func processResponse(url: URL) {
        do {
            var text = ""
            if let transaction = try? ResponseHandler.parsePaymentResult(url: url) {
                text.append(process(payment: transaction))
            } else if let cancellation = try? ResponseHandler.parseCancellationResult(url: url) {
                text.append(process(cancellation: cancellation))
            } else if let dataClearing = try? ResponseHandler.parseDataClearingResult(url: url) {
                text.append(process(dataClearing: dataClearing))
            } else {
                let sync = try ResponseHandler.parseSyncResult(url: url)
                text.append(process(pendingTransaction: sync))
            }
            text.append("\nResponse: \n\(url.absoluteString)\n")

            responseTextView.text = text
        } catch let error {
            var text = "***** ERROR *****\n"
            text.append("Response: \(url.absoluteString)\n")
            text.append("Error: \(error.localizedDescription)")
            responseTextView.text = text
        }
    }

    private func process(payment: PaymentResult) -> String {
        var text = ""
        switch payment.status {
        case .SUCCESS:
            lastTranID = payment.identifier
            lastTranAmount = payment.amount
            updateCancelTransactionButton()
            text.append("Zahlung wurde erfolgreich durchgeführt.\n\n")
            text.append(responseText(payment: payment))
        case .FAILURE:
            text.append("Zahlung fehlgeschlagen.\n\n")
            text.append(responseText(payment: payment))
        case .SIGNATURE_TIME_OUT:
            text.append("Zahlung wurde erfolgreich durchgeführt.\nSignatur wurde nicht eingegeben.\n\n")
            text.append(responseText(payment: payment))
        default:
            // The transaction did not take place
            text.append(couldNotPerformPaymentReasonText(status: payment.status))
            text.append(responseText(payment: payment))
            break
        }

        return text
    }

    private func process(cancellation: CancellationResult) -> String {
        var text = ""
        switch cancellation.status {
        case .SUCCESS:
            lastTranID = nil
            lastTranAmount = nil
            updateCancelTransactionButton()
            text.append("Storno wurde erfolgreich durchgeführt.\n\n")
            text.append(responseText(cancellation: cancellation))
        case .FAILURE:
            text.append("Storno fehlgeschlagen.\n\n")
            text.append(responseText(cancellation: cancellation))
        default:
            // The cancellation did not take place
            text.append(couldNotPerformCancellationReasonText(status: cancellation.status))
            text.append(responseText(cancellation: cancellation))
            break
        }
        return text
    }

    private func process(dataClearing: DataClearingResult) -> String {
        var text = ""
        switch dataClearing.status {
        case .SUCCESS:
            text.append("Kassenschnitt wurde erfolgreich durchgeführt.\n\n")
            text.append(responseText(dataClearing: dataClearing))
        case .FAILURE:
            text.append("Kassenschnitt fehlgeschlagen.\n\n")
            text.append(responseText(dataClearing: dataClearing))
        default:
            // The cancellation did not take place
            text.append(couldNotPerformDataClearingReasonText(status: dataClearing.status))
            text.append(responseText(dataClearing: dataClearing))
            break
        }
        return text
    }

    private func process(pendingTransaction: SyncResult) -> String {
        var text = ""
        switch pendingTransaction.status {
        case .SYNC_OPERATION_SUCCEEDED:
            text.append("Sync wurde erfolgreich durchgeführt.\n\n")
            text.append(responseText(pending: pendingTransaction))
        case .SYNC_OPERATION_FAILED:
            text.append("Sync fehlgeschlagen.\n\n")
            text.append(responseText(pending: pendingTransaction))
        default:
            // The sync did not take place
            text.append(couldNotPerformSyncReasonText(status: pendingTransaction.status))
            text.append(responseText(pending: pendingTransaction))
            break
        }
        return text
    }

}

extension MainViewController: SKStoreProductViewControllerDelegate {
    func productViewControllerDidFinish(_ viewController: SKStoreProductViewController) {
        self.dismiss(animated: true, completion: nil)
    }
}

extension MainViewController {

    fileprivate func setupUserInterface() {
        updateCancelTransactionButton()
        responseTextView.text = ""
    }

    fileprivate func updateCancelTransactionButton() {
        cancelTransactionButton.isEnabled = lastTranID != nil && lastTranAmount != nil
        cancelTransactionButton.alpha = cancelTransactionButton.isEnabled ? 1 : 0.5
    }

    fileprivate func responseText(payment: PaymentResult) -> String {
        var text = ""
        text.append("\nTransaktion-ID: \(payment.identifier)")
        text.append("\nPAN: \(String(describing: payment.maskedPan))")
        text.append("\nKarte: \(String(describing: payment.cardBrand))")
        text.append("\nZahlungsart: \(String(describing: payment.cardType))")
        text.append("\nBetrag: \(payment.amount)")
        text.append("\nMehrwertsteuer: \(payment.tax != nil ? String(describing: payment.tax!) : "")")
        text.append("\nTransaktionsreferenz: \(String(describing: payment.userReference ?? ""))")
        text.append("\nBeleg: \(String(describing: payment.customerReceipt))")
        return text
    }

    fileprivate func couldNotPerformPaymentReasonText(status: PaymentResultStatus) -> String {
        var text = "\nNicht durchgeführt.\n\n"
        text.append("Grund : ")
        switch status {
        case .INVALID_INPUT:
            text.append("Ungültige Anfrage")
        case .NO_CONNECTION:
            text.append("Terminal nicht verbunden")
        case .NOT_READY:
            text.append("Application nicht bereit")
        case .MAIL_NOT_SENT:
            text.append("Mailversand nicht möglich")
        case .SUCCESS:
            break
        case .FAILURE:
            break
        case .SIGNATURE_TIME_OUT:
            break
        case .LIB_VERSION_TOO_LOW:
            text.append("Update der Lib notwendig")
        case .APP_VERSION_TOO_LOW:
            text.append("Update der App notwendig")
        @unknown default:
            break
        }
        return text
    }

    fileprivate func responseText(cancellation: CancellationResult) -> String {
        return ""
    }

    fileprivate func couldNotPerformCancellationReasonText(status: CancellationResultStatus) -> String {
        var text = "\nNicht durchgeführt.\n\n"
        text.append("Grund : ")
        switch status {
        case .INVALID_INPUT:
            text.append("Ungültige Anfrage")
        case .NO_CONNECTION:
            text.append("Terminal nicht verbunden")
        case .NOT_READY:
            text.append("Application nicht bereit")
        case .SUCCESS:
            break
        case .FAILURE:
            break
        case .LIB_VERSION_TOO_LOW:
            text.append("Update der Lib notwendig")
        case .APP_VERSION_TOO_LOW:
            text.append("Update der App notwendig")
        @unknown default:
            break
        }
        return text
    }


    fileprivate func responseText(dataClearing: DataClearingResult) -> String {
        var text = ""
        text.append("\nBeleg: \(dataClearing.receipt ?? "")")
        text.append("\nFreigegebene Transaktionen:\n\(String(describing: dataClearing.clearedIdentifiers ?? []))")
        return text
    }

    fileprivate func couldNotPerformDataClearingReasonText(status: DataClearingResultStatus) -> String {
        var text = "\nNicht durchgeführt.\n\n"
        text.append("Grund : ")
        switch status {
        case .NO_CONNECTION:
            text.append("Terminal nicht verbunden")
        case .NOT_READY:
            text.append("Application nicht bereit")
        case .SUCCESS:
            break
        case .FAILURE:
            break
        case .LIB_VERSION_TOO_LOW:
            text.append("Update der Lib notwendig")
        case .APP_VERSION_TOO_LOW:
            text.append("Update der App notwendig")
        @unknown default:
            break
        }
        return text
    }

    fileprivate func couldNotPerformSyncReasonText(status: SyncResultStatus) -> String {
        var text = "\nNicht durchgeführt.\n\n"
        text.append("Grund : ")
        switch status {
        case .NO_CONNECTION:
            text.append("Terminal nicht verbunden")
        case .NOT_READY:
            text.append("Application nicht bereit")
        case .SYNC_OPERATION_SUCCEEDED:
            break
        case .SYNC_OPERATION_FAILED:
            break
        case .LIB_VERSION_TOO_LOW:
            text.append("Update der Lib notwendig")
        case .APP_VERSION_TOO_LOW:
            text.append("Update der App notwendig")
        @unknown default:
            break
        }
        return text
    }

    fileprivate func responseText(pending: SyncResult) -> String {
        var text = ""
        text.append("\nTransaktion-ID: \(String(describing: pending.payment?.identifier))")
        text.append("\nPAN: \(String(describing: pending.payment?.maskedPan))")
        text.append("\nKarte: \(String(describing: pending.payment?.cardBrand))")
        text.append("\nZahlungsart: \(String(describing: pending.payment?.cardType))")
        text.append("\nBetrag: \(pending.payment?.amount)")
        text.append("\nMehrwertsteuer: \(String(describing: pending.payment?.tax))")
        text.append("\nTransaktionsreferenz: \(String(describing: pending.payment?.userReference ?? ""))")
        text.append("\nBeleg: \(String(describing: pending.dataClearing?.receipt))")
        text.append("\nKunden-Beleg: \(String(describing: pending.payment?.customerReceipt))")
        text.append("\nFreigegebene Transactionen: \(String(describing: pending.dataClearing?.clearedIdentifiers))")
        return text
    }
}
