//
// PaymentViewController.swift
// VRPaymeTestApp
// 
// Created by VR Payment GmbH on 24.09.19
// Copyright © 2019 VR Payment GmbH. All rights reserved.
//

import UIKit
import VRPayme

class PaymentViewController: UIViewController {

    #if OBJC
    var apiService: APIService = OBJCVRPaymeAPIService() as! APIService
    #else
    var apiService: APIService = VRPaymeAPIService()
    #endif

    @IBOutlet var scrollView: UIScrollView!
    @IBOutlet var amountTextField: UITextField!
    @IBOutlet var taxPicker: UIPickerView!
    @IBOutlet var tipTextField: UITextField!
    @IBOutlet var cashierTextField: UITextField!
    @IBOutlet var emailTextField: UITextField!
    @IBOutlet var userReferenceTextField: UITextField!
    @IBOutlet var executePaymentButton: UIButton!

    override func viewDidLoad() {
        super.viewDidLoad()
        setupUserInterface()
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(keyboardWillShow(not:)),
                                               name: UIResponder.keyboardWillShowNotification,
                                               object: nil)
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(keyboardWillHide(not:)),
                                               name: UIResponder.keyboardWillHideNotification,
                                               object: nil)
    }

}

extension PaymentViewController {

    @IBAction func didPressExecutePaymentButton(_ sender: Any) {
        let amount = Int(amountTextField.text ?? "0") ?? 0
        let row = taxPicker.selectedRow(inComponent: 0)
        let tax = row == 0 ? nil : NSNumber(value: TaxRate.allCases[row - 1].rawValue)
        let cashier = cashierTextField.text?.isEmpty == false ? cashierTextField.text : nil
        let email = emailTextField.text?.isEmpty == false ? emailTextField.text! : nil
        let tip = tipTextField.text?.isEmpty == false ? Int(tipTextField.text!) : nil
        let userReference = userReferenceTextField.text?.isEmpty == false ? userReferenceTextField.text! : nil
        do {
            try apiService.performPayment(amount: amount,
                                          tax: tax,
                                          cashier: cashier,
                                          tip: tip ?? 0,
                                          email: email,
                                          userReference: userReference)
            navigationController?.popViewController(animated: true)
        } catch RequestBuilderError.missingVRPaymeApp {
            navigationController?.popViewController(animated: true)
            DispatchQueue.main.async {
                if let parent = self.navigationController?.viewControllers.first as? MainViewController {
                    parent.presentMissingVRPayMeAppAlert()
                }
            }

        } catch let error {
            if let parent = navigationController?.viewControllers.first as? MainViewController {
                parent.updateResponseField(text: error.localizedDescription)
            }
            navigationController?.popViewController(animated: true)
        }

    }

    private func setupUserInterface() {
        executePaymentButton.isEnabled = false
        executePaymentButton.alpha = 0.5
        scrollView.keyboardDismissMode = .onDrag
        let color = UIColor(named: "vrVeryLightBlue") ?? .white
        let attributes: [NSAttributedString.Key: Any] = [NSAttributedString.Key.foregroundColor: color]
        amountTextField.attributedPlaceholder = NSAttributedString(string: "Betrag", attributes: attributes)
        cashierTextField.attributedPlaceholder = NSAttributedString(string: "Kassierername", attributes: attributes)
        tipTextField.attributedPlaceholder = NSAttributedString(string: "Trinkgeld", attributes: attributes)
        emailTextField.attributedPlaceholder = NSAttributedString(string: "E-Mail Adresse für den Beleg",
                                                                  attributes: attributes)
        userReferenceTextField.attributedPlaceholder = NSAttributedString(string: "Transaktionsreferenz",
                                                                          attributes: attributes)

    }

    @objc
    private func keyboardWillShow(not: Notification) {
        let keyboardFrame = not.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as! CGRect
        scrollView.contentInset.bottom = keyboardFrame.height
    }

    @objc
    private func keyboardWillHide(not: Notification) {
        self.scrollView.contentInset.bottom = 0
    }
}

extension PaymentViewController: UITextFieldDelegate {

    func textFieldDidEndEditing(_ textField: UITextField) {
        executePaymentButton.isEnabled = amountTextField.text?.isEmpty == false
        executePaymentButton.alpha = amountTextField.isEnabled ? 1 : 0.5
    }
}

enum TaxRate: Double, CaseIterable {
    case percentage00 = 0.0
    case percentage50 = 5.0
    case percentage55 = 5.5
    case percentage70 = 7.0
    case percentage107 = 10.7
    case percentage160 = 16.0
    case percentage190 = 19.0
}

extension PaymentViewController: UIPickerViewDelegate {

    func pickerView(_ pickerView: UIPickerView, widthForComponent component: Int) -> CGFloat {
        return 40
    }
    func pickerView(_ pickerView: UIPickerView, rowHeightForComponent component: Int) -> CGFloat {
        return 20
    }

    func pickerView(_ pickerView: UIPickerView,
                    viewForRow row: Int,
                    forComponent component: Int,
                    reusing view: UIView?) -> UIView {

        let formatter = NumberFormatter()
        formatter.numberStyle = .percent
        formatter.minimumFractionDigits = 1

        let text = row == 0 ? "-" : formatter.string(from: NSNumber(floatLiteral: TaxRate.allCases[row - 1].rawValue / 100.0))!
        if let view = view as? UILabel {
            view.text = text
            return view
        } else {
            let view = UILabel.init(frame: CGRect(x: 0, y: 0, width: pickerView.frame.width, height: 20))
            view.textColor = UIColor(named: "vrVeryLightBlue") ?? .white
            view.font = UIFont.systemFont(ofSize: 17)
            view.text = text
            view.textAlignment = .right
            return view
        }
    }

    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        row == 0 ? print("-") : print("Selected \(TaxRate.allCases[row - 1])")
    }

}

extension PaymentViewController: UIPickerViewDataSource {

    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }

    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return TaxRate.allCases.count + 1
    }

}
